package org.jboss.seam.cache;

import org.jboss.seam.statistics.StatisticsManager;
import org.jboss.seam.contexts.*;
import org.jboss.seam.mock.MockServletContext;
import org.jboss.seam.mock.MockExternalContext;
import org.jboss.seam.Component;
import org.jboss.seam.Seam;
import org.jboss.seam.init.Initialization;
import org.jboss.seam.core.Init;

import java.util.Random;

/**
 *
 * @author Sebastian Hennebrueder
 */

public class CacheProviderLoadTest implements Runnable {

  private static Random r = new Random();
  private Cache<FileSystemCacheProvider> cacheProvider;

  private static int threadCount = 0;

  private static String contentpostfix = " huhu cache is hopefully working";

  public CacheProviderLoadTest(Cache<FileSystemCacheProvider> cacheProvider) {
    this.cacheProvider = cacheProvider;
  }


  private static void installComponent(Context appContext, Class clazz) {
    appContext.set(Seam.getComponentName(clazz) + ".component",
            new Component(clazz));
  }


  public static void main(String[] args) {

    MockServletContext servletContext = new MockServletContext();
    ServletLifecycle.beginApplication(servletContext);

    MockExternalContext externalContext = new MockExternalContext(
            servletContext);
    Context appContext = new ApplicationContext(externalContext
            .getApplicationMap());
    new Initialization(servletContext).create().init();
    Lifecycle.beginCall();

    long start = System.currentTimeMillis();

    Cache cache = JbossTreeCacheProvider.instance();
//    Cache cache = EhCacheProvider.instance();
    int max = 25;
    int maxThread = 100;

    for (int i = 0; i < max; i++) {
      String key = "key" + r.nextInt(50);


      String content = key + contentpostfix;
      cache.addOrUpdate(key, content);
    }


    for (int i = 0; i < maxThread; i++) {
      Thread thread = new Thread(new CacheProviderLoadTest(cache));
      thread.start();

      threadCount++;
    }

    while (threadCount > 0) {
      try {
        Thread.currentThread().sleep(1000);
      } catch (InterruptedException e) {
        e.printStackTrace();
      }
    }

    System.out.println("report:\n" + StatisticsManager.instance().createReport());

    ServletLifecycle.endApplication();

    System.out.println("time=" + (System.currentTimeMillis() - start));
  }


  /**
   * When an object implementing interface <code>Runnable</code> is used
   * to create a thread, starting the thread causes the object's
   * <code>run</code> method to be called in that separately executing
   * thread.
   * <p/>
   * The general contract of the method <code>run</code> is that it may
   * take any action whatsoever.
   *
   * @see Thread#run()
   */
  public void run() {
    for (int i = 0; i < 500000; i++) {
      String key = "key" + r.nextInt(50);
      Object o = cacheProvider.get(key);
      if (o != null) {
        String c = (String) o;
        int goodhash = (key + contentpostfix).hashCode();
        if (c.hashCode() != goodhash)
          throw new RuntimeException("Bad hash");
      }
      if (i % 20000 == 0) {
        System.out.println("i=" + i);
        try {
          Thread.currentThread().sleep(1000);
        } catch (InterruptedException e) {
          e.printStackTrace();
        }
      }
    }
    threadCount--;
  }
}
