package org.jboss.seam.cache;

import org.testng.annotations.Test;
import org.testng.Assert;
import org.jboss.seam.mock.MockServletContext;
import org.jboss.seam.mock.MockExternalContext;
import org.jboss.seam.contexts.ServletLifecycle;
import org.jboss.seam.contexts.Context;
import org.jboss.seam.contexts.ApplicationContext;
import org.jboss.seam.contexts.Lifecycle;
import org.jboss.seam.init.Initialization;
import org.jboss.seam.Component;

/**
 * Test for the JBoss Cache 1.x
 * Note that the API has largely changed for 2.x
 *
 * @author Sebastian Hennebrueder
 */

public class JbossTreeCacheTest {
  @Test
  public void simple() {

    MockServletContext servletContext = new MockServletContext();
    ServletLifecycle.beginApplication(servletContext);

    MockExternalContext externalContext = new MockExternalContext(
            servletContext);
    Context appContext = new ApplicationContext(externalContext
            .getApplicationMap());

    new Initialization(servletContext).create().init();
    Lifecycle.beginCall();

    Cache cache = (Cache) Component.getInstance(JbossTreeCacheProvider.class);
    String key = "myKey";
    Assert.assertNull(cache.get(key));

    cache.addOrUpdate(key, "myContent");
    Assert.assertEquals("myContent", cache.get(key));

    cache.clear();
    Assert.assertNull(cache.get(key));

    cache.addOrUpdate(key, "myContent");
    Assert.assertEquals("myContent", cache.get(key));
    ServletLifecycle.endApplication();
  }
}
